( cd usr/lib ; rm -rf libextractor.so.1 )
( cd usr/lib ; ln -sf libextractor.so.1.1.1 libextractor.so.1 )
( cd usr/lib ; rm -rf libextractor.so )
( cd usr/lib ; ln -sf libextractor.so.1.1.1 libextractor.so )
( cd usr/lib/libextractor ; rm -rf libextractor_thumbnail.so )
( cd usr/lib/libextractor ; ln -sf /usr/lib/libextractor/libextractor_thumbnailgtk.so libextractor_thumbnail.so )
